@@ -1,6 +1,6 @@
 package overload;
 
-our $VERSION = '1.02';
+our $VERSION = '1.03';
 
 $overload::hint_bits = 0x20000; # HINT_LOCALIZE_HH
 
@@ -93,11 +93,7 @@ sub AddrRef {
 	return sprintf("$class_prefix$type(0x%x)", $addr);
 }
 
-sub StrVal {
-  (ref $_[0] && OverloadedStringify($_[0]) or ref($_[0]) eq 'Regexp') ?
-    (AddrRef(shift)) :
-    "$_[0]";
-}
+*StrVal = *AddrRef;
 
 sub mycan {				# Real can would leave stubs.
   my ($package, $meth) = @_;
