@@ -4,9 +4,8 @@ _o='.o'
 afs='false'
 afsroot='/afs'
 alignbytes='4'
-apiversion='5.005'
-archlib='/usr/local/lib/perl5/5.7/unknown'
-archlibexp='/usr/local/lib/perl5/5.7/unknown'
+archlib='/usr/local/lib/perl5/5.9/unknown'
+archlibexp='/usr/local/lib/perl5/5.9/unknown'
 archname='unknown'
 asctime_r_proto='0'
 bin='/usr/local/bin'
@@ -598,8 +597,8 @@ orderlib='false'
 osname='unknown'
 phostname='hostname'
 pidtype=int
-privlib='/usr/local/lib/perl5/5.7'
-privlibexp='/usr/local/lib/perl5/5.7'
+privlib='/usr/local/lib/perl5/5.9'
+privlibexp='/usr/local/lib/perl5/5.9'
 procselfexe=''
 prototype='undef'
 ptrsize='4'
@@ -645,7 +644,7 @@ sig_num_init='0'
 sig_size='1'
 signal_t=int
 sizesize=4
-sizetype=int
+sizetype='size_t'
 socksizetype='int'
 srand48_r_proto='0'
 srandom_r_proto='0'
@@ -684,6 +683,7 @@ usefaststdio='undef'
 useithreads='undef'
 uselargefiles='undef'
 uselongdouble='undef'
+usemallocwrap='undef'
 usemorebits='undef'
 usemultiplicity='undef'
 usemymalloc='n'
